@@ -1,9 +1,10 @@
 <?php
 
-/**
+/*
  +-----------------------------------------------------------------------+
- | This file is part of the Roundcube PHP suite                          |
- | Copyright (C) 2005-2017, The Roundcube Dev Team                       |
+ | This file is part of the Roundcube webmail client                     |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -17,46 +18,45 @@
  +-----------------------------------------------------------------------+
 */
 
-/**
- * Roundcube Framework Initialization
- *
- * @package    Framework
- * @subpackage Core
- */
-
-$config = array(
-    'error_reporting'         => E_ALL & ~E_NOTICE & ~E_STRICT,
-    // Some users are not using Installer, so we'll check some
-    // critical PHP settings here. Only these, which doesn't provide
-    // an error/warning in the logs later. See (#1486307).
-    'mbstring.func_overload'  => 0,
-    'magic_quotes_runtime'    => false,
-    'magic_quotes_sybase'     => false, // #1488506
-);
+// Some users are not using Installer, so we'll check some critical PHP settings here
+$config = [
+    'display_errors' => false,
+    'log_errors' => true,
+];
 
 // check these additional ini settings if not called via CLI
-if (php_sapi_name() != 'cli') {
-    $config += array(
+if (\PHP_SAPI != 'cli') {
+    $config += [
         'suhosin.session.encrypt' => false,
-        'file_uploads'            => true,
-    );
+        'file_uploads' => true,
+        'session.auto_start' => false,
+        'zlib.output_compression' => false,
+    ];
 }
 
 foreach ($config as $optname => $optval) {
-    $ini_optval = filter_var(ini_get($optname), is_bool($optval) ? FILTER_VALIDATE_BOOLEAN : FILTER_VALIDATE_INT);
+    // @phpstan-ignore-next-line
+    $ini_optval = filter_var(ini_get($optname), is_bool($optval) ? \FILTER_VALIDATE_BOOLEAN : \FILTER_VALIDATE_INT);
     if ($optval != $ini_optval && @ini_set($optname, $optval) === false) {
+        // @phpstan-ignore-next-line
         $optval = !is_bool($optval) ? $optval : ($optval ? 'On' : 'Off');
-        $error  = "ERROR: Wrong '$optname' option value and it wasn't possible to set it to required value ($optval).\n"
-            . "Check your PHP configuration (including php_admin_flag).";
+        $error = "ERROR: Wrong '{$optname}' option value and it wasn't possible to set it to required value ({$optval}).\n"
+            . 'Check your PHP configuration (including php_admin_flag).';
+
+        if (defined('STDERR')) {
+            fwrite(\STDERR, $error);
+        } else {
+            echo $error;
+        }
 
-        if (defined('STDERR')) fwrite(STDERR, $error); else echo $error;
         exit(1);
     }
 }
 
 // framework constants
-define('RCUBE_VERSION', '1.2.7');
+define('RCUBE_VERSION', '1.7-rc');
 define('RCUBE_CHARSET', 'UTF-8');
+define('RCUBE_TEMP_FILE_PREFIX', 'RCMTEMP');
 
 if (!defined('RCUBE_LIB_DIR')) {
     define('RCUBE_LIB_DIR', __DIR__ . '/');
@@ -79,52 +79,49 @@ if (!defined('RCUBE_LOCALIZATION_DIR')) {
 }
 
 // set internal encoding for mbstring extension
-if (function_exists('mb_internal_encoding')) {
-    mb_internal_encoding(RCUBE_CHARSET);
-}
-if (function_exists('mb_regex_encoding')) {
-    mb_regex_encoding(RCUBE_CHARSET);
-}
+mb_internal_encoding(RCUBE_CHARSET);
+mb_regex_encoding(RCUBE_CHARSET);
 
 // make sure the Roundcube lib directory is in the include_path
 $rcube_path = realpath(RCUBE_LIB_DIR . '..');
-$sep        = PATH_SEPARATOR;
-$regexp     = "!(^|$sep)" . preg_quote($rcube_path, '!') . "($sep|\$)!";
-$path       = ini_get('include_path');
+$sep = \PATH_SEPARATOR;
+$regexp = "!(^|{$sep})" . preg_quote($rcube_path, '!') . "({$sep}|\$)!";
+$path = ini_get('include_path');
 
 if (!preg_match($regexp, $path)) {
-    set_include_path($path . PATH_SEPARATOR . $rcube_path);
+    set_include_path($path . \PATH_SEPARATOR . $rcube_path);
 }
 
 // Register autoloader
 spl_autoload_register('rcube_autoload');
 
 // set PEAR error handling (will also load the PEAR main class)
-PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'rcube_pear_error');
-
+if (class_exists('PEAR')) {
+    // @phpstan-ignore-next-line
+    \PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, static function ($err) { rcube::raise_error($err, true); });
+}
 
 /**
  * Similar function as in_array() but case-insensitive with multibyte support.
  *
  * @param string $needle   Needle value
- * @param array  $heystack Array to search in
+ * @param ?array $haystack Array to search in
  *
- * @return boolean True if found, False if not
+ * @return bool True if found, False if not
  */
 function in_array_nocase($needle, $haystack)
 {
     // use much faster method for ascii
     if (is_ascii($needle)) {
         foreach ((array) $haystack as $value) {
-            if (strcasecmp($value, $needle) === 0) {
+            if (is_string($value) && strcasecmp($value, $needle) === 0) {
                 return true;
             }
         }
-    }
-    else {
+    } else {
         $needle = mb_strtolower($needle);
         foreach ((array) $haystack as $value) {
-            if ($needle === mb_strtolower($value)) {
+            if (is_string($value) && $needle === mb_strtolower($value)) {
                 return true;
             }
         }
@@ -136,57 +133,60 @@ function in_array_nocase($needle, $haystack)
 /**
  * Parse a human readable string for a number of bytes.
  *
- * @param string $str Input string
+ * @param string|int|float $str Input string
  *
- * @return float Number of bytes
+ * @return int|false Number of bytes
  */
 function parse_bytes($str)
 {
-    if (is_numeric($str)) {
-        return floatval($str);
-    }
-
-    if (preg_match('/([0-9\.]+)\s*([a-z]*)/i', $str, $regs)) {
+    if (preg_match('/^([0-9\.]+)\s*([KMGT]?)I?B?$/', trim(strtoupper((string) $str)), $regs)) {
         $bytes = floatval($regs[1]);
-        switch (strtolower($regs[2])) {
-        case 'g':
-        case 'gb':
-            $bytes *= 1073741824;
-            break;
-        case 'm':
-        case 'mb':
-            $bytes *= 1048576;
-            break;
-        case 'k':
-        case 'kb':
-            $bytes *= 1024;
-            break;
+        switch ($regs[2]) {
+            case 'T':
+                $bytes *= 1024;
+            case 'G':
+                $bytes *= 1024;
+            case 'M':
+                $bytes *= 1024;
+            case 'K':
+                $bytes *= 1024;
+                break;
         }
+
+        return (int) round($bytes);
     }
 
-    return floatval($bytes);
+    return false;
 }
 
 /**
  * Make sure the string ends with a slash
+ *
+ * @param string $str A string
+ *
+ * @return string A string ending with a slash
  */
 function slashify($str)
 {
-  return unslashify($str).'/';
+    return unslashify($str) . '/';
 }
 
 /**
  * Remove slashes at the end of the string
+ *
+ * @param string $str A string
+ *
+ * @return string A string ending with no slash
  */
 function unslashify($str)
 {
-  return preg_replace('/\/+$/', '', $str);
+    return rtrim($str, '/');
 }
 
 /**
  * Returns number of seconds for a specified offset string.
  *
- * @param string $str String representation of the offset (e.g. 20min, 5h, 2days, 1week)
+ * @param string|int $str String representation of the offset (e.g. 20min, 5h, 2days, 1week)
  *
  * @return int Number of seconds
  */
@@ -194,22 +194,21 @@ function get_offset_sec($str)
 {
     if (preg_match('/^([0-9]+)\s*([smhdw])/i', $str, $regs)) {
         $amount = (int) $regs[1];
-        $unit   = strtolower($regs[2]);
-    }
-    else {
+        $unit = strtolower($regs[2]);
+    } else {
         $amount = (int) $str;
-        $unit   = 's';
+        $unit = 's';
     }
 
     switch ($unit) {
-    case 'w':
-        $amount *= 7;
-    case 'd':
-        $amount *= 24;
-    case 'h':
-        $amount *= 60;
-    case 'm':
-        $amount *= 60;
+        case 'w':
+            $amount *= 7;
+        case 'd':
+            $amount *= 24;
+        case 'h':
+            $amount *= 60;
+        case 'm':
+            $amount *= 60;
     }
 
     return $amount;
@@ -229,8 +228,8 @@ function get_offset_time($offset_str, $factor = 1)
 }
 
 /**
- * Truncate string if it is longer than the allowed length.
- * Replace the middle or the ending part of a string with a placeholder.
+ * Truncates a string if it is longer than the allowed length. Replaces
+ * the middle or the ending part of a string with a placeholder.
  *
  * @param string $str         Input string
  * @param int    $maxlength   Max. length
@@ -249,12 +248,12 @@ function abbreviate_string($str, $maxlength, $placeholder = '...', $ending = fal
         }
 
         $placeholder_length = mb_strlen($placeholder);
-        $first_part_length  = floor(($maxlength - $placeholder_length)/2);
+        $first_part_length = floor(($maxlength - $placeholder_length) / 2);
         $second_starting_location = $length - $maxlength + $first_part_length + $placeholder_length;
 
         $prefix = mb_substr($str, 0, $first_part_length);
         $suffix = mb_substr($str, $second_starting_location);
-        $str    = $prefix . $placeholder . $suffix;
+        $str = $prefix . $placeholder . $suffix;
     }
 
     return $str;
@@ -269,9 +268,10 @@ function abbreviate_string($str, $maxlength, $placeholder = '...', $ending = fal
  */
 function array_keys_recursive($array)
 {
-    $keys = array();
+    $keys = [];
 
-    if (!empty($array) && is_array($array)) {
+    // @phpstan-ignore-next-line
+    if (is_array($array)) {
         foreach ($array as $key => $child) {
             $keys[] = $key;
             foreach (array_keys_recursive($child) as $val) {
@@ -285,11 +285,17 @@ function array_keys_recursive($array)
 
 /**
  * Remove all non-ascii and non-word chars except ., -, _
+ *
+ * @param string $str          A string
+ * @param bool   $css_id       The result may be used as CSS identifier
+ * @param string $replace_with Replacement character
+ *
+ * @return string Clean string
  */
 function asciiwords($str, $css_id = false, $replace_with = '')
 {
     $allowed = 'a-z0-9\_\-' . (!$css_id ? '\.' : '');
-    return preg_replace("/[^$allowed]/i", $replace_with, $str);
+    return preg_replace("/[^{$allowed}]+/i", $replace_with, (string) $str);
 }
 
 /**
@@ -298,12 +304,12 @@ function asciiwords($str, $css_id = false, $replace_with = '')
  * @param string $str           String to check
  * @param bool   $control_chars Includes control characters
  *
- * @return bool
+ * @return bool True if the string contains ASCII-only, False otherwise
  */
 function is_ascii($str, $control_chars = true)
 {
     $regexp = $control_chars ? '/[^\x00-\x7F]/' : '/[^\x20-\x7E]/';
-    return preg_match($regexp, $str) ? false : true;
+    return preg_match($regexp, (string) $str) ? false : true;
 }
 
 /**
@@ -321,10 +327,10 @@ function format_email_recipient($email, $name = '')
     if ($name && $name != $email) {
         // Special chars as defined by RFC 822 need to in quoted string (or escaped).
         if (preg_match('/[\(\)\<\>\\\.\[\]@,;:"]/', $name)) {
-            $name = '"'.addcslashes($name, '"').'"';
+            $name = '"' . addcslashes($name, '"') . '"';
         }
 
-        return "$name <$email>";
+        return "{$name} <{$email}>";
     }
 
     return $email;
@@ -344,7 +350,7 @@ function format_email($email)
     $count = count($parts);
 
     if ($count > 1) {
-        $parts[$count-1] = mb_strtolower($parts[$count-1]);
+        $parts[$count - 1] = mb_strtolower($parts[$count - 1]);
 
         $email = implode('@', $parts);
     }
@@ -357,110 +363,46 @@ function format_email($email)
  *
  * @param string $version Version number string
  *
- * @param return Version number string
+ * @return string Version number string
  */
 function version_parse($version)
 {
     return str_replace(
-        array('-stable', '-git'),
-        array('.0', '.99'),
+        ['-stable', '-git'],
+        ['.0', '.99'],
         $version
     );
 }
 
-/**
- * intl replacement functions
- */
-
-if (!function_exists('idn_to_utf8'))
-{
-    function idn_to_utf8($domain)
-    {
-        static $idn, $loaded;
-
-        if (!$loaded) {
-            $idn    = new Net_IDNA2();
-            $loaded = true;
-        }
-
-        if ($idn && $domain && preg_match('/(^|\.)xn--/i', $domain)) {
-            try {
-                $domain = $idn->decode($domain);
-            }
-            catch (Exception $e) {
-            }
-        }
-
-        return $domain;
-    }
-}
-
-if (!function_exists('idn_to_ascii'))
-{
-    function idn_to_ascii($domain)
-    {
-        static $idn, $loaded;
-
-        if (!$loaded) {
-            $idn    = new Net_IDNA2();
-            $loaded = true;
-        }
-
-        if ($idn && $domain && preg_match('/[^\x20-\x7E]/', $domain)) {
-            try {
-                $domain = $idn->encode($domain);
-            }
-            catch (Exception $e) {
-            }
-        }
-
-        return $domain;
-    }
-}
-
 /**
  * Use PHP5 autoload for dynamic class loading
  *
- * @todo Make Zend, PEAR etc play with this
- * @todo Make our classes conform to a more straight forward CS.
+ * @return bool True when the class file has been found
  */
-function rcube_autoload($classname)
+function rcube_autoload(string $classname): bool
 {
-    if (strpos($classname, 'rcube') === 0) {
+    if (str_starts_with($classname, 'rcube')) {
+        $classname = preg_replace('/^rcube_(cache|db|session|spellchecker)_/', '\1/', $classname);
         $classname = 'Roundcube/' . $classname;
-    }
-    else if (strpos($classname, 'html_') === 0 || $classname === 'html') {
+    } elseif (str_starts_with($classname, 'html_') || $classname === 'html') {
         $classname = 'Roundcube/html';
-    }
-    else if (strpos($classname, 'Mail_') === 0) {
+    } elseif (str_starts_with($classname, 'Mail_')) {
         $classname = 'Mail/' . substr($classname, 5);
-    }
-    else if (strpos($classname, 'Net_') === 0) {
+    } elseif (str_starts_with($classname, 'Net_')) {
         $classname = 'Net/' . substr($classname, 4);
-    }
-    else if (strpos($classname, 'Auth_') === 0) {
+    } elseif (str_starts_with($classname, 'Auth_')) {
         $classname = 'Auth/' . substr($classname, 5);
     }
 
-    if ($fp = @fopen("$classname.php", 'r', true)) {
+    // Translate PHP namespaces into directories,
+    // i.e. 'Sabre\Reader' -> 'Sabre/Reader.php'
+    $classname = str_replace('\\', '/', $classname);
+
+    if ($fp = @fopen("{$classname}.php", 'r', true)) {
         fclose($fp);
-        include_once "$classname.php";
+        include_once "{$classname}.php";
         return true;
     }
 
     return false;
 }
-
-/**
- * Local callback function for PEAR errors
- */
-function rcube_pear_error($err)
-{
-    $msg = sprintf("ERROR: %s (%s)", $err->getMessage(), $err->getCode());
-
-    if ($info = $err->getUserinfo()) {
-        $msg .= ': ' . $info;
-    }
-
-    error_log($msg, 0);
-}
